# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Code that's shared between multiple security policies subcommands."""

import base64
import json

from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import yaml
from googlecloudsdk.core.resource import resource_printer
import six


def SecurityPolicyFromFile(input_file, messages, file_format):
  """Returns the security policy read from the given file.

  Args:
    input_file: file, A file with a security policy config.
    messages: messages, The set of available messages.
    file_format: string, the format of the file to read from

  Returns:
    A security policy resource.
  """

  if file_format == 'yaml':
    parsed_security_policy = yaml.load(input_file)
  else:
    try:
      parsed_security_policy = json.load(input_file)
    except ValueError as e:
      raise exceptions.BadFileException('Error parsing JSON: {0}'.format(
          six.text_type(e)))

  security_policy = messages.SecurityPolicy()
  if 'shortName' in parsed_security_policy:
    security_policy.shortName = parsed_security_policy['shortName']
  if 'description' in parsed_security_policy:
    security_policy.description = parsed_security_policy['description']
  if 'fingerprint' in parsed_security_policy:
    security_policy.fingerprint = base64.urlsafe_b64decode(
        parsed_security_policy['fingerprint'].encode('ascii'))
  if 'type' in parsed_security_policy:
    security_policy.type = (
        messages.SecurityPolicy.TypeValueValuesEnum(
            parsed_security_policy['type']))
  if 'cloudArmorConfig' in parsed_security_policy:
    security_policy.cloudArmorConfig = messages.SecurityPolicyCloudArmorConfig(
        enableMl=parsed_security_policy['cloudArmorConfig']['enableMl'])
  if 'adaptiveProtectionConfig' in parsed_security_policy:
    security_policy.adaptiveProtectionConfig = (
        messages.SecurityPolicyAdaptiveProtectionConfig(
            layer7DdosDefenseConfig=messages
            .SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig(
                enable=parsed_security_policy['adaptiveProtectionConfig']
                ['layer7DdosDefenseConfig']['enable']),))
    for parsed_threshold_config in parsed_security_policy[
        'adaptiveProtectionConfig'
    ]['layer7DdosDefenseConfig'].get('thresholdConfigs', []):
      threshold_config = messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfig(
          name=parsed_threshold_config['name']
      )
      if 'autoDeployConfidenceThreshold' in parsed_threshold_config:
        threshold_config.autoDeployConfidenceThreshold = (
            parsed_threshold_config['autoDeployConfidenceThreshold']
        )
      if 'autoDeployExpirationSec' in parsed_threshold_config:
        threshold_config.autoDeployExpirationSec = parsed_threshold_config[
            'autoDeployExpirationSec'
        ]
      if 'autoDeployImpactedBaselineThreshold' in parsed_threshold_config:
        threshold_config.autoDeployImpactedBaselineThreshold = (
            parsed_threshold_config['autoDeployImpactedBaselineThreshold']
        )
      if 'autoDeployLoadThreshold' in parsed_threshold_config:
        threshold_config.autoDeployLoadThreshold = parsed_threshold_config[
            'autoDeployLoadThreshold'
        ]
      if 'detectionAbsoluteQps' in parsed_threshold_config:
        threshold_config.detectionAbsoluteQps = parsed_threshold_config[
            'detectionAbsoluteQps'
        ]
      if 'detectionLoadThreshold' in parsed_threshold_config:
        threshold_config.detectionLoadThreshold = parsed_threshold_config[
            'detectionLoadThreshold'
        ]
      if 'detectionRelativeToBaselineQps' in parsed_threshold_config:
        threshold_config.detectionRelativeToBaselineQps = (
            parsed_threshold_config['detectionRelativeToBaselineQps']
        )
      for parsed_traffic_granularity_config in parsed_threshold_config.get(
          'trafficGranularityConfigs', []
      ):
        traffic_granularity_config = (
            messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfigTrafficGranularityConfig()
        )
        if 'enableEachUniqueValue' in parsed_traffic_granularity_config:
          traffic_granularity_config.enableEachUniqueValue = (
              parsed_traffic_granularity_config['enableEachUniqueValue']
          )
        if 'type' in parsed_traffic_granularity_config:
          traffic_granularity_config.type = messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfigTrafficGranularityConfig.TypeValueValuesEnum(
              parsed_traffic_granularity_config['type']
          )
        if 'value' in parsed_traffic_granularity_config:
          traffic_granularity_config.value = parsed_traffic_granularity_config[
              'value'
          ]
        threshold_config.trafficGranularityConfigs.append(
            traffic_granularity_config
        )

      security_policy.adaptiveProtectionConfig.layer7DdosDefenseConfig.thresholdConfigs.append(
          threshold_config
      )

    if 'autoDeployConfig' in parsed_security_policy['adaptiveProtectionConfig']:
      security_policy.adaptiveProtectionConfig.autoDeployConfig = (
          messages.SecurityPolicyAdaptiveProtectionConfigAutoDeployConfig())
      if 'loadThreshold' in parsed_security_policy['adaptiveProtectionConfig'][
          'autoDeployConfig']:
        security_policy.adaptiveProtectionConfig.autoDeployConfig.loadThreshold = (
            parsed_security_policy['adaptiveProtectionConfig']
            ['autoDeployConfig']['loadThreshold'])
      if 'confidenceThreshold' in parsed_security_policy[
          'adaptiveProtectionConfig']['autoDeployConfig']:
        security_policy.adaptiveProtectionConfig.autoDeployConfig.confidenceThreshold = (
            parsed_security_policy['adaptiveProtectionConfig']
            ['autoDeployConfig']['confidenceThreshold'])
      if 'impactedBaselineThreshold' in parsed_security_policy[
          'adaptiveProtectionConfig']['autoDeployConfig']:
        security_policy.adaptiveProtectionConfig.autoDeployConfig.impactedBaselineThreshold = (
            parsed_security_policy['adaptiveProtectionConfig']
            ['autoDeployConfig']['impactedBaselineThreshold'])
      if 'expirationSec' in parsed_security_policy['adaptiveProtectionConfig'][
          'autoDeployConfig']:
        security_policy.adaptiveProtectionConfig.autoDeployConfig.expirationSec = (
            parsed_security_policy['adaptiveProtectionConfig']
            ['autoDeployConfig']['expirationSec'])
    if 'ruleVisibility' in parsed_security_policy['adaptiveProtectionConfig'][
        'layer7DdosDefenseConfig']:
      security_policy.adaptiveProtectionConfig.layer7DdosDefenseConfig.ruleVisibility = (
          messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig
          .RuleVisibilityValueValuesEnum(
              parsed_security_policy['adaptiveProtectionConfig']
              ['layer7DdosDefenseConfig']['ruleVisibility']))
  if 'advancedOptionsConfig' in parsed_security_policy:
    advanced_options_config = parsed_security_policy['advancedOptionsConfig']
    security_policy.advancedOptionsConfig = (
        messages.SecurityPolicyAdvancedOptionsConfig())
    if 'jsonParsing' in advanced_options_config:
      security_policy.advancedOptionsConfig.jsonParsing = (
          messages.SecurityPolicyAdvancedOptionsConfig
          .JsonParsingValueValuesEnum(
              advanced_options_config['jsonParsing']))
    if 'jsonCustomConfig' in advanced_options_config:
      security_policy.advancedOptionsConfig.jsonCustomConfig = (
          messages.SecurityPolicyAdvancedOptionsConfigJsonCustomConfig(
              contentTypes=advanced_options_config
              ['jsonCustomConfig'].get('contentTypes', [])))
    if 'logLevel' in advanced_options_config:
      security_policy.advancedOptionsConfig.logLevel = (
          messages.SecurityPolicyAdvancedOptionsConfig.LogLevelValueValuesEnum(
              advanced_options_config['logLevel']))
    if 'requestBodyInspectionSize' in advanced_options_config:
      security_policy.advancedOptionsConfig.requestBodyInspectionSize = (
          advanced_options_config['requestBodyInspectionSize']
      )
    if 'userIpRequestHeaders' in advanced_options_config:
      security_policy.advancedOptionsConfig.userIpRequestHeaders = (
          advanced_options_config['userIpRequestHeaders'])
  if 'ddosProtectionConfig' in parsed_security_policy:
    security_policy.ddosProtectionConfig = (
        messages.SecurityPolicyDdosProtectionConfig(
            ddosProtection=messages.SecurityPolicyDdosProtectionConfig
            .DdosProtectionValueValuesEnum(
                parsed_security_policy['ddosProtectionConfig']
                ['ddosProtection'])))
    if (
        'ddosAdaptiveProtection'
        in parsed_security_policy['ddosProtectionConfig']
    ):
      security_policy.ddosProtectionConfig.ddosAdaptiveProtection = messages.SecurityPolicyDdosProtectionConfig.DdosAdaptiveProtectionValueValuesEnum(
          parsed_security_policy['ddosProtectionConfig'][
              'ddosAdaptiveProtection'
          ]
      )
    if (
        'ddosImpactedBaselineThreshold'
        in parsed_security_policy['ddosProtectionConfig']
    ):
      security_policy.ddosProtectionConfig.ddosImpactedBaselineThreshold = (
          parsed_security_policy['ddosProtectionConfig'][
              'ddosImpactedBaselineThreshold'
          ]
      )
  if 'recaptchaOptionsConfig' in parsed_security_policy:
    security_policy.recaptchaOptionsConfig = (
        messages.SecurityPolicyRecaptchaOptionsConfig())
    if 'redirectSiteKey' in parsed_security_policy['recaptchaOptionsConfig']:
      security_policy.recaptchaOptionsConfig.redirectSiteKey = (
          parsed_security_policy['recaptchaOptionsConfig']['redirectSiteKey'])

  if 'userDefinedFields' in parsed_security_policy:
    user_defined_fields = []
    for udf in parsed_security_policy['userDefinedFields']:
      user_defined_field = messages.SecurityPolicyUserDefinedField()
      user_defined_field.name = udf['name']
      user_defined_field.base = (
          messages.SecurityPolicyUserDefinedField.BaseValueValuesEnum(
              udf['base']
          )
      )
      user_defined_field.offset = udf['offset']
      user_defined_field.size = udf['size']
      if 'mask' in udf:
        user_defined_field.mask = udf['mask']
      user_defined_fields.append(user_defined_field)
    security_policy.userDefinedFields = user_defined_fields

  rules = []
  if 'rules' in parsed_security_policy:
    for rule in parsed_security_policy['rules']:
      security_policy_rule = messages.SecurityPolicyRule()
      security_policy_rule.action = rule['action']
      if 'description' in rule:
        security_policy_rule.description = rule['description']
      if 'match' in rule:
        match = messages.SecurityPolicyRuleMatcher()
        if 'versionedExpr' in rule['match']:
          match.versionedExpr = ConvertToEnum(
              rule['match']['versionedExpr'], messages
          )
        if 'expr' in rule['match']:
          match.expr = messages.Expr(
              expression=rule['match']['expr']['expression']
          )
        if 'exprOptions' in rule['match']:
          expr_options = messages.SecurityPolicyRuleMatcherExprOptions()
          if 'recaptchaOptions' in rule['match']['exprOptions']:
            expr_options.recaptchaOptions = (
                messages.SecurityPolicyRuleMatcherExprOptionsRecaptchaOptions(
                    actionTokenSiteKeys=rule['match']['exprOptions'][
                        'recaptchaOptions'
                    ].get('actionTokenSiteKeys', []),
                    sessionTokenSiteKeys=rule['match']['exprOptions'][
                        'recaptchaOptions'
                    ].get('sessionTokenSiteKeys', []),
                )
            )
          match.exprOptions = expr_options
        if 'config' in rule['match']:
          if 'srcIpRanges' in rule['match']['config']:
            match.config = messages.SecurityPolicyRuleMatcherConfig(
                srcIpRanges=rule['match']['config']['srcIpRanges']
            )
        security_policy_rule.match = match
      if 'networkMatch' in rule:
        network_match = messages.SecurityPolicyRuleNetworkMatcher()
        if 'userDefinedFields' in rule['networkMatch']:
          user_defined_fields = []
          for udf in rule['networkMatch']['userDefinedFields']:
            user_defined_field_match = (
                messages.SecurityPolicyRuleNetworkMatcherUserDefinedFieldMatch()
            )
            user_defined_field_match.name = udf['name']
            user_defined_field_match.values = udf['values']
            user_defined_fields.append(user_defined_field_match)
          network_match.userDefinedFields = user_defined_fields
        if 'srcIpRanges' in rule['networkMatch']:
          network_match.srcIpRanges = rule['networkMatch']['srcIpRanges']
        if 'destIpRanges' in rule['networkMatch']:
          network_match.destIpRanges = rule['networkMatch']['destIpRanges']
        if 'ipProtocols' in rule['networkMatch']:
          network_match.ipProtocols = rule['networkMatch']['ipProtocols']
        if 'srcPorts' in rule['networkMatch']:
          network_match.srcPorts = rule['networkMatch']['srcPorts']
        if 'destPorts' in rule['networkMatch']:
          network_match.destPorts = rule['networkMatch']['destPorts']
        if 'srcRegionCodes' in rule['networkMatch']:
          network_match.srcRegionCodes = rule['networkMatch']['srcRegionCodes']
        if 'srcAsns' in rule['networkMatch']:
          network_match.srcAsns = rule['networkMatch']['srcAsns']
        security_policy_rule.networkMatch = network_match
      security_policy_rule.priority = int(rule['priority'])
      if 'preview' in rule:
        security_policy_rule.preview = rule['preview']
      rules.append(security_policy_rule)
      if 'redirectTarget' in rule:
        security_policy_rule.redirectTarget = rule['redirectTarget']
      if 'ruleNumber' in rule:
        security_policy_rule.ruleNumber = int(rule['ruleNumber'])
      if 'redirectOptions' in rule:
        redirect_options = messages.SecurityPolicyRuleRedirectOptions()
        if 'type' in rule['redirectOptions']:
          redirect_options.type = (
              messages.SecurityPolicyRuleRedirectOptions.TypeValueValuesEnum(
                  rule['redirectOptions']['type']
              )
          )
        if 'target' in rule['redirectOptions']:
          redirect_options.target = rule['redirectOptions']['target']
        security_policy_rule.redirectOptions = redirect_options
      if 'headerAction' in rule:
        header_action = messages.SecurityPolicyRuleHttpHeaderAction()
        headers_in_rule = rule['headerAction'].get('requestHeadersToAdds', [])
        headers_to_add = []
        for header_to_add in headers_in_rule:
          headers_to_add.append(
              messages.SecurityPolicyRuleHttpHeaderActionHttpHeaderOption(
                  headerName=header_to_add['headerName'],
                  headerValue=header_to_add['headerValue'],
              )
          )
        if headers_to_add:
          header_action.requestHeadersToAdds = headers_to_add
        security_policy_rule.headerAction = header_action
      if 'rateLimitOptions' in rule:
        rate_limit_options = rule['rateLimitOptions']
        security_policy_rule.rateLimitOptions = messages.SecurityPolicyRuleRateLimitOptions(
            rateLimitThreshold=messages.SecurityPolicyRuleRateLimitOptionsThreshold(
                count=rate_limit_options['rateLimitThreshold']['count'],
                intervalSec=rate_limit_options['rateLimitThreshold'][
                    'intervalSec'
                ],
            ),
            conformAction=rate_limit_options['conformAction'],
            exceedAction=rate_limit_options['exceedAction'],
        )
        if 'exceedActionRpcStatus' in rate_limit_options:
          exceed_action_rpc_status = (
              messages.SecurityPolicyRuleRateLimitOptionsRpcStatus()
          )
          if 'code' in rate_limit_options['exceedActionRpcStatus']:
            exceed_action_rpc_status.code = rate_limit_options[
                'exceedActionRpcStatus'
            ]['code']
          if 'message' in rate_limit_options['exceedActionRpcStatus']:
            exceed_action_rpc_status.message = rate_limit_options[
                'exceedActionRpcStatus'
            ]['message']
          security_policy_rule.rateLimitOptions.exceedActionRpcStatus = (
              exceed_action_rpc_status
          )
        if 'exceedRedirectOptions' in rate_limit_options:
          exceed_redirect_options = messages.SecurityPolicyRuleRedirectOptions()
          if 'type' in rate_limit_options['exceedRedirectOptions']:
            exceed_redirect_options.type = (
                messages.SecurityPolicyRuleRedirectOptions.TypeValueValuesEnum(
                    rate_limit_options['exceedRedirectOptions']['type']
                )
            )
          if 'target' in rate_limit_options['exceedRedirectOptions']:
            exceed_redirect_options.target = rate_limit_options[
                'exceedRedirectOptions'
            ]['target']
          security_policy_rule.rateLimitOptions.exceedRedirectOptions = (
              exceed_redirect_options
          )
        if 'banThreshold' in rate_limit_options:
          security_policy_rule.rateLimitOptions.banThreshold = (
              messages.SecurityPolicyRuleRateLimitOptionsThreshold(
                  count=rate_limit_options['banThreshold']['count'],
                  intervalSec=rate_limit_options['banThreshold']['intervalSec'],
              )
          )
        if 'banDurationSec' in rate_limit_options:
          security_policy_rule.rateLimitOptions.banDurationSec = (
              rate_limit_options['banDurationSec']
          )
        if 'enforceOnKey' in rate_limit_options:
          security_policy_rule.rateLimitOptions.enforceOnKey = messages.SecurityPolicyRuleRateLimitOptions.EnforceOnKeyValueValuesEnum(
              rate_limit_options['enforceOnKey']
          )
        if 'enforceOnKeyName' in rate_limit_options:
          security_policy_rule.rateLimitOptions.enforceOnKeyName = (
              rate_limit_options['enforceOnKeyName']
          )
        for config in rate_limit_options.get('enforceOnKeyConfigs', []):
          enforce_on_key_config = (
              messages.SecurityPolicyRuleRateLimitOptionsEnforceOnKeyConfig()
          )
          if 'enforceOnKeyType' in config:
            enforce_on_key_config.enforceOnKeyType = messages.SecurityPolicyRuleRateLimitOptionsEnforceOnKeyConfig.EnforceOnKeyTypeValueValuesEnum(
                config['enforceOnKeyType']
            )
          if 'enforceOnKeyName' in config:
            enforce_on_key_config.enforceOnKeyName = config['enforceOnKeyName']
          security_policy_rule.rateLimitOptions.enforceOnKeyConfigs.append(
              enforce_on_key_config
          )
      if 'preconfiguredWafConfig' in rule:
        preconfig_waf_config = (
            messages.SecurityPolicyRulePreconfiguredWafConfig()
        )
        for exclusion in rule['preconfiguredWafConfig'].get('exclusions', []):
          exclusion_to_add = (
              messages.SecurityPolicyRulePreconfiguredWafConfigExclusion()
          )
          if 'targetRuleSet' in exclusion:
            exclusion_to_add.targetRuleSet = exclusion['targetRuleSet']
          for target_rule_id in exclusion.get('targetRuleIds', []):
            exclusion_to_add.targetRuleIds.append(target_rule_id)
          for request_header in exclusion.get('requestHeadersToExclude', []):
            exclusion_to_add.requestHeadersToExclude.append(
                ConvertPreconfigWafExclusionRequestField(
                    request_header, messages
                )
            )
          for request_cookie in exclusion.get('requestCookiesToExclude', []):
            exclusion_to_add.requestCookiesToExclude.append(
                ConvertPreconfigWafExclusionRequestField(
                    request_cookie, messages
                )
            )
          for request_query_param in exclusion.get(
              'requestQueryParamsToExclude', []
          ):
            exclusion_to_add.requestQueryParamsToExclude.append(
                ConvertPreconfigWafExclusionRequestField(
                    request_query_param, messages
                )
            )
          for request_uri in exclusion.get('requestUrisToExclude', []):
            exclusion_to_add.requestUrisToExclude.append(
                ConvertPreconfigWafExclusionRequestField(request_uri, messages)
            )
          if hasattr(exclusion_to_add, 'requestBodiesToExclude'):
            for request_body in exclusion.get('requestBodiesToExclude', []):
              exclusion_to_add.requestBodiesToExclude.append(
                  ConvertPreconfigWafExclusionRequestField(
                      request_body, messages
                  )
              )
          preconfig_waf_config.exclusions.append(exclusion_to_add)
        security_policy_rule.preconfiguredWafConfig = preconfig_waf_config

  security_policy.rules = rules

  return security_policy


def ConvertToEnum(versioned_expr, messages):
  """Converts a string version of a versioned expr to the enum representation.

  Args:
    versioned_expr: string, string version of versioned expr to convert.
    messages: messages, The set of available messages.

  Returns:
    A versioned expression enum.
  """
  return messages.SecurityPolicyRuleMatcher.VersionedExprValueValuesEnum(
      versioned_expr)


def ConvertPreconfigWafExclusionRequestField(request_field_in_rule, messages):
  """Converts the request field in preconfigured WAF exclusion configuration.

  Args:
    request_field_in_rule: a request field in preconfigured WAF exclusion
      configuration read from the security policy config file.
    messages: the set of available messages.

  Returns:
    The proto representation of the request field.
  """
  request_field = (
      messages.SecurityPolicyRulePreconfiguredWafConfigExclusionFieldParams())
  if 'op' in request_field_in_rule:
    request_field.op = (
        messages.SecurityPolicyRulePreconfiguredWafConfigExclusionFieldParams
        .OpValueValuesEnum(request_field_in_rule['op']))
  if 'val' in request_field_in_rule:
    request_field.val = request_field_in_rule['val']
  return request_field


def WriteToFile(output_file, security_policy, file_format):
  """Writes the given security policy in the given format to the given file.

  Args:
    output_file: file, File into which the security policy should be written.
    security_policy: resource, SecurityPolicy to be written out.
    file_format: string, the format of the file to write to.
  """
  resource_printer.Print(
      security_policy, print_format=file_format, out=output_file)


def CreateCloudArmorConfig(client, args):
  """Returns a SecurityPolicyCloudArmorConfig message."""

  messages = client.messages
  cloud_armor_config = None
  if args.enable_ml is not None:
    cloud_armor_config = messages.SecurityPolicyCloudArmorConfig(
        enableMl=args.enable_ml)
  return cloud_armor_config


def CreateAdaptiveProtectionConfig(client, args,
                                   existing_adaptive_protection_config):
  """Returns a SecurityPolicyAdaptiveProtectionConfig message."""

  messages = client.messages
  adaptive_protection_config = (
      existing_adaptive_protection_config if existing_adaptive_protection_config
      is not None else messages.SecurityPolicyAdaptiveProtectionConfig())

  if (args.IsSpecified('enable_layer7_ddos_defense') or
      args.IsSpecified('layer7_ddos_defense_rule_visibility')):
    layer7_ddos_defense_config = (
        (adaptive_protection_config.layer7DdosDefenseConfig)
        if adaptive_protection_config.layer7DdosDefenseConfig is not None
        else messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig()
    )
    if args.IsSpecified('enable_layer7_ddos_defense'):
      layer7_ddos_defense_config.enable = args.enable_layer7_ddos_defense
    if args.IsSpecified('layer7_ddos_defense_rule_visibility'):
      layer7_ddos_defense_config.ruleVisibility = (
          messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig
          .RuleVisibilityValueValuesEnum(
              args.layer7_ddos_defense_rule_visibility))
    adaptive_protection_config.layer7DdosDefenseConfig = (
        layer7_ddos_defense_config)

  return adaptive_protection_config


def CreateAdaptiveProtectionConfigWithAutoDeployConfig(
    client, args, existing_adaptive_protection_config):
  """Returns a SecurityPolicyAdaptiveProtectionConfig message with AutoDeployConfig."""

  messages = client.messages
  adaptive_protection_config = CreateAdaptiveProtectionConfig(
      client, args, existing_adaptive_protection_config)

  if args.IsSpecified(
      'layer7_ddos_defense_auto_deploy_load_threshold') or args.IsSpecified(
          'layer7_ddos_defense_auto_deploy_confidence_threshold'
      ) or args.IsSpecified(
          'layer7_ddos_defense_auto_deploy_impacted_baseline_threshold'
      ) or args.IsSpecified('layer7_ddos_defense_auto_deploy_expiration_sec'):
    auto_deploy_config = (
        adaptive_protection_config.autoDeployConfig
        if adaptive_protection_config.autoDeployConfig is not None else
        messages.SecurityPolicyAdaptiveProtectionConfigAutoDeployConfig())
    if args.IsSpecified('layer7_ddos_defense_auto_deploy_load_threshold'):
      auto_deploy_config.loadThreshold = (
          args.layer7_ddos_defense_auto_deploy_load_threshold)
    if args.IsSpecified('layer7_ddos_defense_auto_deploy_confidence_threshold'):
      auto_deploy_config.confidenceThreshold = (
          args.layer7_ddos_defense_auto_deploy_confidence_threshold)
    if args.IsSpecified(
        'layer7_ddos_defense_auto_deploy_impacted_baseline_threshold'):
      auto_deploy_config.impactedBaselineThreshold = (
          args.layer7_ddos_defense_auto_deploy_impacted_baseline_threshold)
    if args.IsSpecified('layer7_ddos_defense_auto_deploy_expiration_sec'):
      auto_deploy_config.expirationSec = (
          args.layer7_ddos_defense_auto_deploy_expiration_sec)

    adaptive_protection_config.autoDeployConfig = auto_deploy_config

  return adaptive_protection_config


def CreateAdvancedOptionsConfig(
    client, args, existing_advanced_options_config, enable_large_body_size=False
):
  """Returns a SecurityPolicyAdvancedOptionsConfig message."""

  messages = client.messages
  advanced_options_config = (
      existing_advanced_options_config if existing_advanced_options_config
      is not None else messages.SecurityPolicyAdvancedOptionsConfig())

  if args.IsSpecified('json_parsing'):
    advanced_options_config.jsonParsing = (
        messages.SecurityPolicyAdvancedOptionsConfig.JsonParsingValueValuesEnum(
            args.json_parsing))

  if args.IsSpecified('json_custom_content_types'):
    advanced_options_config.jsonCustomConfig = (
        messages.SecurityPolicyAdvancedOptionsConfigJsonCustomConfig(
            contentTypes=args.json_custom_content_types))

  if args.IsSpecified('log_level'):
    advanced_options_config.logLevel = (
        messages.SecurityPolicyAdvancedOptionsConfig.LogLevelValueValuesEnum(
            args.log_level))

  if enable_large_body_size and args.IsSpecified(
      'request_body_inspection_size'
  ):
    advanced_options_config.requestBodyInspectionSize = (
        args.request_body_inspection_size
    )

  if args.IsSpecified('user_ip_request_headers'):
    advanced_options_config.userIpRequestHeaders = args.user_ip_request_headers

  return advanced_options_config


def CreateDdosProtectionConfig(client, args, existing_ddos_protection_config):
  """Returns a SecurityPolicyDdosProtectionConfig message."""

  messages = client.messages
  ddos_protection_config = (
      existing_ddos_protection_config if existing_ddos_protection_config
      is not None else messages.SecurityPolicyDdosProtectionConfig())

  if args.IsSpecified('network_ddos_protection'):
    ddos_protection_config.ddosProtection = (
        messages.SecurityPolicyDdosProtectionConfig
        .DdosProtectionValueValuesEnum(args.network_ddos_protection))

  return ddos_protection_config


def CreateDdosProtectionConfigWithDdosAdaptiveProtection(
    client, args, existing_ddos_protection_config
):
  """Returns a SecurityPolicyDdosProtectionConfig message."""

  messages = client.messages
  ddos_protection_config = (
      existing_ddos_protection_config
      if existing_ddos_protection_config is not None
      else messages.SecurityPolicyDdosProtectionConfig()
  )
  if args.IsSpecified('network_ddos_adaptive_protection'):
    ddos_protection_config.ddosAdaptiveProtection = messages.SecurityPolicyDdosProtectionConfig.DdosAdaptiveProtectionValueValuesEnum(
        args.network_ddos_adaptive_protection
    )

  return ddos_protection_config


def CreateDdosProtectionConfigWithNetworkDdosImpactedBaselineThreshold(
    client, args, existing_ddos_protection_config
):
  """Returns a SecurityPolicyDdosProtectionConfig message."""

  messages = client.messages
  ddos_protection_config = (
      existing_ddos_protection_config
      if existing_ddos_protection_config is not None
      else messages.SecurityPolicyDdosProtectionConfig()
  )
  if args.IsSpecified('network_ddos_impacted_baseline_threshold'):
    ddos_protection_config.ddosImpactedBaselineThreshold = (
        args.network_ddos_impacted_baseline_threshold
    )

  return ddos_protection_config


def CreateDdosProtectionConfigOld(client, args,
                                  existing_ddos_protection_config):
  """Returns a SecurityPolicyDdosProtectionConfig message."""

  messages = client.messages
  ddos_protection_config = (
      existing_ddos_protection_config if existing_ddos_protection_config
      is not None else messages.SecurityPolicyDdosProtectionConfig())

  if args.IsSpecified('ddos_protection'):
    ddos_protection_config.ddosProtection = (
        messages.SecurityPolicyDdosProtectionConfig
        .DdosProtectionValueValuesEnum(args.ddos_protection))

  return ddos_protection_config


def CreateRecaptchaOptionsConfig(client, args,
                                 existing_recaptcha_options_config):
  """Returns a SecurityPolicyRecaptchaOptionsConfig message."""

  messages = client.messages
  recaptcha_options_config = (
      existing_recaptcha_options_config if existing_recaptcha_options_config
      is not None else messages.SecurityPolicyRecaptchaOptionsConfig())

  if args.IsSpecified('recaptcha_redirect_site_key'):
    recaptcha_options_config.redirectSiteKey = args.recaptcha_redirect_site_key

  return recaptcha_options_config


def ParseEnforceOnKeyConfig(config):
  """Parses an enforce-on-key-config string into a key-value tuple."""
  if '=' in config:
    k, v = config.split('=', 1)
    return k, v
  else:
    return config, None


def CreateRateLimitOptions(client, args, support_fairshare):
  """Returns a SecurityPolicyRuleRateLimitOptions message."""

  messages = client.messages
  rate_limit_options = messages.SecurityPolicyRuleRateLimitOptions()
  is_updated = False

  if (args.IsSpecified('rate_limit_threshold_count') or
      args.IsSpecified('rate_limit_threshold_interval_sec')):
    rate_limit_threshold = (
        messages.SecurityPolicyRuleRateLimitOptionsThreshold())
    if args.IsSpecified('rate_limit_threshold_count'):
      rate_limit_threshold.count = args.rate_limit_threshold_count
    if args.IsSpecified('rate_limit_threshold_interval_sec'):
      rate_limit_threshold.intervalSec = args.rate_limit_threshold_interval_sec
    rate_limit_options.rateLimitThreshold = rate_limit_threshold
    is_updated = True

  if args.IsSpecified('conform_action'):
    rate_limit_options.conformAction = _ConvertConformAction(
        args.conform_action)
    is_updated = True
  if args.IsSpecified('exceed_action'):
    rate_limit_options.exceedAction = _ConvertExceedAction(args.exceed_action)
    is_updated = True
  if (args.IsSpecified('exceed_redirect_type') or
      args.IsSpecified('exceed_redirect_target')):
    exceed_redirect_options = messages.SecurityPolicyRuleRedirectOptions()
    if args.IsSpecified('exceed_redirect_type'):
      exceed_redirect_options.type = (
          messages.SecurityPolicyRuleRedirectOptions.TypeValueValuesEnum(
              _ConvertRedirectType(args.exceed_redirect_type)))
    if args.IsSpecified('exceed_redirect_target'):
      exceed_redirect_options.target = args.exceed_redirect_target
    rate_limit_options.exceedRedirectOptions = exceed_redirect_options
    is_updated = True
  if args.IsSpecified('enforce_on_key'):
    rate_limit_options.enforceOnKey = (
        messages.SecurityPolicyRuleRateLimitOptions.EnforceOnKeyValueValuesEnum(
            ConvertEnforceOnKey(args.enforce_on_key)))
    is_updated = True
  if args.IsSpecified('enforce_on_key_name'):
    rate_limit_options.enforceOnKeyName = args.enforce_on_key_name
    is_updated = True

  if args.IsSpecified('enforce_on_key_configs'):
    enforce_on_key_configs = []
    for k, v in args.enforce_on_key_configs:
      enforce_on_key_configs.append(
          messages.SecurityPolicyRuleRateLimitOptionsEnforceOnKeyConfig(
              enforceOnKeyType=messages
              .SecurityPolicyRuleRateLimitOptionsEnforceOnKeyConfig
              .EnforceOnKeyTypeValueValuesEnum(ConvertEnforceOnKey(k)),
              enforceOnKeyName=v,
          ))
    rate_limit_options.enforceOnKeyConfigs = enforce_on_key_configs
    is_updated = True

  if (args.IsSpecified('ban_threshold_count') or
      args.IsSpecified('ban_threshold_interval_sec')):
    ban_threshold = messages.SecurityPolicyRuleRateLimitOptionsThreshold()
    if args.IsSpecified('ban_threshold_count'):
      ban_threshold.count = args.ban_threshold_count
    if args.IsSpecified('ban_threshold_interval_sec'):
      ban_threshold.intervalSec = args.ban_threshold_interval_sec
    rate_limit_options.banThreshold = ban_threshold
    is_updated = True

  if args.IsSpecified('ban_duration_sec'):
    rate_limit_options.banDurationSec = args.ban_duration_sec
    is_updated = True

  if support_fairshare and (
      args.IsSpecified('exceed_action_rpc_status_code') or
      args.IsSpecified('exceed_action_rpc_status_message')):
    exceed_action_rpc_status = (
        messages.SecurityPolicyRuleRateLimitOptionsRpcStatus()
    )
    if args.IsSpecified('exceed_action_rpc_status_code'):
      exceed_action_rpc_status.code = args.exceed_action_rpc_status_code
    if args.IsSpecified('exceed_action_rpc_status_message'):
      exceed_action_rpc_status.message = args.exceed_action_rpc_status_message
    rate_limit_options.exceedActionRpcStatus = exceed_action_rpc_status
    is_updated = True

  return rate_limit_options if is_updated else None


def _ConvertConformAction(action):
  return {'allow': 'allow'}.get(action, action)


def _ConvertExceedAction(action):
  return {
      'deny-403': 'deny(403)',
      'deny-404': 'deny(404)',
      'deny-429': 'deny(429)',
      'deny-502': 'deny(502)'
  }.get(action, action)


def ConvertEnforceOnKey(enforce_on_key):
  return {
      'ip': 'IP',
      'all-ips': 'ALL_IPS',
      'all': 'ALL',
      'http-header': 'HTTP_HEADER',
      'xff-ip': 'XFF_IP',
      'http-cookie': 'HTTP_COOKIE',
      'http-path': 'HTTP_PATH',
      'sni': 'SNI',
      'region-code': 'REGION_CODE',
      'tls-ja3-fingerprint': 'TLS_JA3_FINGERPRINT',
      'user-ip': 'USER_IP',
      'tls-ja4-fingerprint': 'TLS_JA4_FINGERPRINT'
  }.get(enforce_on_key, enforce_on_key)


def CreateRedirectOptions(client, args):
  """Returns a SecurityPolicyRuleRedirectOptions message."""

  messages = client.messages
  redirect_options = messages.SecurityPolicyRuleRedirectOptions()
  is_updated = False

  if args.IsSpecified('redirect_type'):
    redirect_options.type = (
        messages.SecurityPolicyRuleRedirectOptions.TypeValueValuesEnum(
            _ConvertRedirectType(args.redirect_type)))
    is_updated = True

  # If --redirect-target is given while --redirect-type is unspecified, type
  # is implicitly set to be EXTERNAL_302.
  if args.IsSpecified('redirect_target'):
    redirect_options.target = args.redirect_target
    if redirect_options.type is None:
      redirect_options.type = (
          messages.SecurityPolicyRuleRedirectOptions.TypeValueValuesEnum
          .EXTERNAL_302)
    is_updated = True

  return redirect_options if is_updated else None


def _ConvertRedirectType(redirect_type):
  return {
      'google-recaptcha': 'GOOGLE_RECAPTCHA',
      'external-302': 'EXTERNAL_302'
  }.get(redirect_type, redirect_type)


def CreateExpressionOptions(client, args):
  """Returns a SecurityPolicyRuleMatcherExprOptions message."""
  if not args.IsSpecified(
      'recaptcha_action_site_keys'
  ) and not args.IsSpecified('recaptcha_session_site_keys'):
    return None

  messages = client.messages
  recaptcha_options = (
      messages.SecurityPolicyRuleMatcherExprOptionsRecaptchaOptions()
  )

  if args.IsSpecified('recaptcha_action_site_keys'):
    recaptcha_options.actionTokenSiteKeys = args.recaptcha_action_site_keys
  if args.IsSpecified('recaptcha_session_site_keys'):
    recaptcha_options.sessionTokenSiteKeys = args.recaptcha_session_site_keys

  return messages.SecurityPolicyRuleMatcherExprOptions(
      recaptchaOptions=recaptcha_options
  )


def CreateNetworkMatcher(client, args):
  """Returns a SecurityPolicyRuleNetworkMatcher message."""

  messages = client.messages
  network_matcher = messages.SecurityPolicyRuleNetworkMatcher()
  update_mask = []
  is_updated = False

  if getattr(args, 'network_user_defined_fields', None) is not None:
    user_defined_fields = []
    for user_defined_field in args.network_user_defined_fields:
      # user defined field match will be in the following format:
      # "name;value1:value2:value3"
      parsed = user_defined_field.split(';')
      name = parsed[0]
      values = parsed[1].split(':')
      user_defined_fields.append(
          messages.SecurityPolicyRuleNetworkMatcherUserDefinedFieldMatch(
              name=name, values=values
          )
      )
    network_matcher.userDefinedFields = user_defined_fields
    update_mask.append('network_match.user_defined_fields')
    is_updated = True
  if getattr(args, 'network_src_ip_ranges', None) is not None:
    network_matcher.srcIpRanges = args.network_src_ip_ranges
    update_mask.append('network_match.src_ip_ranges')
    is_updated = True
  if getattr(args, 'network_dest_ip_ranges', None) is not None:
    network_matcher.destIpRanges = args.network_dest_ip_ranges
    update_mask.append('network_match.dest_ip_ranges')
    is_updated = True
  if getattr(args, 'network_ip_protocols', None) is not None:
    network_matcher.ipProtocols = args.network_ip_protocols
    update_mask.append('network_match.ip_protocols')
    is_updated = True
  if getattr(args, 'network_src_ports', None) is not None:
    network_matcher.srcPorts = args.network_src_ports
    update_mask.append('network_match.src_ports')
    is_updated = True
  if getattr(args, 'network_dest_ports', None) is not None:
    network_matcher.destPorts = args.network_dest_ports
    update_mask.append('network_match.dest_ports')
    is_updated = True
  if getattr(args, 'network_src_region_codes', None) is not None:
    network_matcher.srcRegionCodes = args.network_src_region_codes
    update_mask.append('network_match.src_region_codes')
    is_updated = True
  if getattr(args, 'network_src_asns', None) is not None:
    network_matcher.srcAsns = [int(asn) for asn in args.network_src_asns]
    update_mask.append('network_match.src_asns')
    is_updated = True

  return (network_matcher, update_mask) if is_updated else (None, [])


def CreateUserDefinedField(client, args):
  """Returns a SecurityPolicyUserDefinedField message."""

  messages = client.messages
  user_defined_field = messages.SecurityPolicyUserDefinedField()

  user_defined_field.name = args.user_defined_field_name
  user_defined_field.base = (
      messages.SecurityPolicyUserDefinedField.BaseValueValuesEnum(
          _ConvertUserDefinedFieldBase(args.base)
      )
  )
  user_defined_field.offset = args.offset
  user_defined_field.size = args.size

  if getattr(args, 'mask', None) is not None:
    user_defined_field.mask = args.mask

  return user_defined_field


def _ConvertUserDefinedFieldBase(base):
  return {'ipv4': 'IPV4', 'ipv6': 'IPV6', 'tcp': 'TCP', 'udp': 'UDP'}.get(
      base, base
  )


def CreateLayer7DdosDefenseThresholdConfig(
    client, args, support_granularity_config
):
  """Returns a SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfig message."""

  messages = client.messages
  threshold_config = (
      messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfig()
  )

  threshold_config.name = args.threshold_config_name

  if args.IsSpecified('auto_deploy_load_threshold'):
    threshold_config.autoDeployLoadThreshold = args.auto_deploy_load_threshold
  if args.IsSpecified('auto_deploy_confidence_threshold'):
    threshold_config.autoDeployConfidenceThreshold = (
        args.auto_deploy_confidence_threshold
    )
  if args.IsSpecified('auto_deploy_impacted_baseline_threshold'):
    threshold_config.autoDeployImpactedBaselineThreshold = (
        args.auto_deploy_impacted_baseline_threshold
    )
  if args.IsSpecified('auto_deploy_expiration_sec'):
    threshold_config.autoDeployExpirationSec = args.auto_deploy_expiration_sec

  if support_granularity_config:
    if args.IsSpecified('detection_load_threshold'):
      threshold_config.detectionLoadThreshold = args.detection_load_threshold
    if args.IsSpecified('detection_absolute_qps'):
      threshold_config.detectionAbsoluteQps = args.detection_absolute_qps
    if args.IsSpecified('detection_relative_to_baseline_qps'):
      threshold_config.detectionRelativeToBaselineQps = (
          args.detection_relative_to_baseline_qps
      )
    if args.IsSpecified('traffic_granularity_configs'):
      traffic_granularity_configs = []
      for arg_traffic_granularity_config in args.traffic_granularity_configs:
        traffic_granularity_config = (
            messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfigTrafficGranularityConfig()
        )
        if 'type' in arg_traffic_granularity_config:
          traffic_granularity_config.type = messages.SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfigThresholdConfigTrafficGranularityConfig.TypeValueValuesEnum(
              arg_traffic_granularity_config['type']
          )
        if 'value' in arg_traffic_granularity_config:
          traffic_granularity_config.value = arg_traffic_granularity_config[
              'value'
          ]
        if 'enableEachUniqueValue' in arg_traffic_granularity_config:
          traffic_granularity_config.enableEachUniqueValue = (
              arg_traffic_granularity_config['enableEachUniqueValue']
          )
        traffic_granularity_configs.append(traffic_granularity_config)
      threshold_config.trafficGranularityConfigs = traffic_granularity_configs

  return threshold_config



"""Generated message classes for apphub version v1.

App Hub lets you build, operate, and manage applications on Google Cloud.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'apphub'


class ApphubProjectsLocationsApplicationsCreateRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsCreateRequest object.

  Fields:
    application: A Application resource to be passed as the request body.
    applicationId: Required. The Application identifier. Must contain only
      lowercase letters, numbers or hyphens, with the first character a
      letter, the last a letter or a number, and a 63 character maximum.
    parent: Required. Project and location to create Application in. Expected
      format: `projects/{project}/locations/{location}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  application = _messages.MessageField('Application', 1)
  applicationId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class ApphubProjectsLocationsApplicationsDeleteRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsDeleteRequest object.

  Fields:
    name: Required. Fully qualified name of the Application to delete.
      Expected format:
      `projects/{project}/locations/{location}/applications/{application}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class ApphubProjectsLocationsApplicationsGetIamPolicyRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsGetIamPolicyRequest object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class ApphubProjectsLocationsApplicationsGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsGetRequest object.

  Fields:
    name: Required. Fully qualified name of the Application to fetch. Expected
      format:
      `projects/{project}/locations/{location}/applications/{application}`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsApplicationsListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsListRequest object.

  Fields:
    filter: Optional. Filtering results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Project and location to list Applications on. Expected
      format: `projects/{project}/locations/{location}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ApphubProjectsLocationsApplicationsPatchRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsPatchRequest object.

  Fields:
    application: A Application resource to be passed as the request body.
    name: Identifier. The resource name of an Application. Format:
      `"projects/{host-project-
      id}/locations/{location}/applications/{application-id}"`
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the Application resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. The API changes the values of the fields as specified in the
      update_mask. The API ignores the values of all fields not covered by the
      update_mask. You can also unset a field by not specifying it in the
      updated message, but adding the field to the mask. This clears whatever
      value the field previously had.
  """

  application = _messages.MessageField('Application', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class ApphubProjectsLocationsApplicationsServicesCreateRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsServicesCreateRequest object.

  Fields:
    parent: Required. Fully qualified name of the parent Application to create
      the Service in. Expected format:
      `projects/{project}/locations/{location}/applications/{application}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    service: A Service resource to be passed as the request body.
    serviceId: Required. The Service identifier. Must contain only lowercase
      letters, numbers or hyphens, with the first character a letter, the last
      a letter or a number, and a 63 character maximum.
  """

  parent = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  service = _messages.MessageField('Service', 3)
  serviceId = _messages.StringField(4)


class ApphubProjectsLocationsApplicationsServicesDeleteRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsServicesDeleteRequest object.

  Fields:
    name: Required. Fully qualified name of the Service to delete from an
      Application. Expected format: `projects/{project}/locations/{location}/a
      pplications/{application}/services/{service}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class ApphubProjectsLocationsApplicationsServicesGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsServicesGetRequest object.

  Fields:
    name: Required. Fully qualified name of the Service to fetch. Expected
      format: `projects/{project}/locations/{location}/applications/{applicati
      on}/services/{service}`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsApplicationsServicesListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsServicesListRequest object.

  Fields:
    filter: Optional. Filtering results
    orderBy: Optional. Hint for how to order the results
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Fully qualified name of the parent Application to list
      Services for. Expected format:
      `projects/{project}/locations/{location}/applications/{application}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ApphubProjectsLocationsApplicationsServicesPatchRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsServicesPatchRequest object.

  Fields:
    name: Identifier. The resource name of a Service. Format:
      `"projects/{host-project-
      id}/locations/{location}/applications/{application-
      id}/services/{service-id}"`
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    service: A Service resource to be passed as the request body.
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the Service resource by the update. The fields specified
      in the update_mask are relative to the resource, not the full request.
      The API changes the values of the fields as specified in the
      update_mask. The API ignores the values of all fields not covered by the
      update_mask. You can also unset a field by not specifying it in the
      updated message, but adding the field to the mask. This clears whatever
      value the field previously had.
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  service = _messages.MessageField('Service', 3)
  updateMask = _messages.StringField(4)


class ApphubProjectsLocationsApplicationsSetIamPolicyRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsSetIamPolicyRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class ApphubProjectsLocationsApplicationsTestIamPermissionsRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsTestIamPermissionsRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class ApphubProjectsLocationsApplicationsWorkloadsCreateRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsWorkloadsCreateRequest object.

  Fields:
    parent: Required. Fully qualified name of the Application to create
      Workload in. Expected format:
      `projects/{project}/locations/{location}/applications/{application}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    workload: A Workload resource to be passed as the request body.
    workloadId: Required. The Workload identifier. Must contain only lowercase
      letters, numbers or hyphens, with the first character a letter, the last
      a letter or a number, and a 63 character maximum.
  """

  parent = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  workload = _messages.MessageField('Workload', 3)
  workloadId = _messages.StringField(4)


class ApphubProjectsLocationsApplicationsWorkloadsDeleteRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsWorkloadsDeleteRequest object.

  Fields:
    name: Required. Fully qualified name of the Workload to delete from an
      Application. Expected format: `projects/{project}/locations/{location}/a
      pplications/{application}/workloads/{workload}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class ApphubProjectsLocationsApplicationsWorkloadsGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsWorkloadsGetRequest object.

  Fields:
    name: Required. Fully qualified name of the Workload to fetch. Expected
      format: `projects/{project}/locations/{location}/applications/{applicati
      on}/workloads/{workload}`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsApplicationsWorkloadsListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsWorkloadsListRequest object.

  Fields:
    filter: Optional. Filtering results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Fully qualified name of the parent Application to list
      Workloads for. Expected format:
      `projects/{project}/locations/{location}/applications/{application}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ApphubProjectsLocationsApplicationsWorkloadsPatchRequest(_messages.Message):
  r"""A ApphubProjectsLocationsApplicationsWorkloadsPatchRequest object.

  Fields:
    name: Identifier. The resource name of the Workload. Format:
      `"projects/{host-project-
      id}/locations/{location}/applications/{application-
      id}/workloads/{workload-id}"`
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the Workload resource by the update. The fields specified
      in the update_mask are relative to the resource, not the full request.
      The API changes the values of the fields as specified in the
      update_mask. The API ignores the values of all fields not covered by the
      update_mask. You can also unset a field by not specifying it in the
      updated message, but adding the field to the mask. This clears whatever
      value the field previously had.
    workload: A Workload resource to be passed as the request body.
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  updateMask = _messages.StringField(3)
  workload = _messages.MessageField('Workload', 4)


class ApphubProjectsLocationsDetachServiceProjectAttachmentRequest(_messages.Message):
  r"""A ApphubProjectsLocationsDetachServiceProjectAttachmentRequest object.

  Fields:
    detachServiceProjectAttachmentRequest: A
      DetachServiceProjectAttachmentRequest resource to be passed as the
      request body.
    name: Required. Service project id and location to detach from a host
      project. Only global location is supported. Expected format:
      `projects/{project}/locations/{location}`.
  """

  detachServiceProjectAttachmentRequest = _messages.MessageField('DetachServiceProjectAttachmentRequest', 1)
  name = _messages.StringField(2, required=True)


class ApphubProjectsLocationsDiscoveredServicesGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsDiscoveredServicesGetRequest object.

  Fields:
    name: Required. Fully qualified name of the Discovered Service to fetch.
      Expected format: `projects/{project}/locations/{location}/discoveredServ
      ices/{discoveredService}`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsDiscoveredServicesListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsDiscoveredServicesListRequest object.

  Fields:
    filter: Optional. Filtering results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Project and location to list Discovered Services on.
      Expected format: `projects/{project}/locations/{location}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ApphubProjectsLocationsDiscoveredServicesLookupRequest(_messages.Message):
  r"""A ApphubProjectsLocationsDiscoveredServicesLookupRequest object.

  Fields:
    parent: Required. Host project ID and location to lookup Discovered
      Service in. Expected format: `projects/{project}/locations/{location}`.
    uri: Required. Resource URI to find DiscoveredService for. Accepts both
      project number and project ID and does translation when needed.
  """

  parent = _messages.StringField(1, required=True)
  uri = _messages.StringField(2)


class ApphubProjectsLocationsDiscoveredWorkloadsGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsDiscoveredWorkloadsGetRequest object.

  Fields:
    name: Required. Fully qualified name of the Discovered Workload to fetch.
      Expected format: `projects/{project}/locations/{location}/discoveredWork
      loads/{discoveredWorkload}`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsDiscoveredWorkloadsListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsDiscoveredWorkloadsListRequest object.

  Fields:
    filter: Optional. Filtering results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Project and location to list Discovered Workloads on.
      Expected format: `projects/{project}/locations/{location}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ApphubProjectsLocationsDiscoveredWorkloadsLookupRequest(_messages.Message):
  r"""A ApphubProjectsLocationsDiscoveredWorkloadsLookupRequest object.

  Fields:
    parent: Required. Host project ID and location to lookup Discovered
      Workload in. Expected format: `projects/{project}/locations/{location}`.
    uri: Required. Resource URI to find Discovered Workload for. Accepts both
      project number and project ID and does translation when needed.
  """

  parent = _messages.StringField(1, required=True)
  uri = _messages.StringField(2)


class ApphubProjectsLocationsExtendedMetadataSchemasGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsExtendedMetadataSchemasGetRequest object.

  Fields:
    name: Required. Schema resource name. Format: `projects/{project}/location
      s/{location}/extendedMetadataSchemas/{extended_metadata_schema}`.
      `{extended_metadata_schema}` has the format
      `"apphub.googleapis.com/{SchemaName}"`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsExtendedMetadataSchemasListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsExtendedMetadataSchemasListRequest object.

  Fields:
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Project and location to list Extended Metadata Schemas
      on. Expected format: `projects/{project}/locations/{location}`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class ApphubProjectsLocationsGetBoundaryRequest(_messages.Message):
  r"""A ApphubProjectsLocationsGetBoundaryRequest object.

  Fields:
    name: Required. The name of the boundary to retrieve. Format:
      `projects/{project}/locations/{location}/boundary`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class ApphubProjectsLocationsLookupServiceProjectAttachmentRequest(_messages.Message):
  r"""A ApphubProjectsLocationsLookupServiceProjectAttachmentRequest object.

  Fields:
    name: Required. Service project ID and location to lookup service project
      attachment for. Only global location is supported. Expected format:
      `projects/{project}/locations/{location}`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A ApphubProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class ApphubProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A ApphubProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class ApphubProjectsLocationsServiceProjectAttachmentsCreateRequest(_messages.Message):
  r"""A ApphubProjectsLocationsServiceProjectAttachmentsCreateRequest object.

  Fields:
    parent: Required. Host project ID and location to which service project is
      being attached. Only global location is supported. Expected format:
      `projects/{project}/locations/{location}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    serviceProjectAttachment: A ServiceProjectAttachment resource to be passed
      as the request body.
    serviceProjectAttachmentId: Required. The service project attachment
      identifier must contain the project id of the service project specified
      in the service_project_attachment.service_project field.
  """

  parent = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  serviceProjectAttachment = _messages.MessageField('ServiceProjectAttachment', 3)
  serviceProjectAttachmentId = _messages.StringField(4)


class ApphubProjectsLocationsServiceProjectAttachmentsDeleteRequest(_messages.Message):
  r"""A ApphubProjectsLocationsServiceProjectAttachmentsDeleteRequest object.

  Fields:
    name: Required. Fully qualified name of the service project attachment to
      delete. Expected format: `projects/{project}/locations/{location}/servic
      eProjectAttachments/{serviceProjectAttachment}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class ApphubProjectsLocationsServiceProjectAttachmentsGetRequest(_messages.Message):
  r"""A ApphubProjectsLocationsServiceProjectAttachmentsGetRequest object.

  Fields:
    name: Required. Fully qualified name of the service project attachment to
      retrieve. Expected format: `projects/{project}/locations/{location}/serv
      iceProjectAttachments/{serviceProjectAttachment}`.
  """

  name = _messages.StringField(1, required=True)


class ApphubProjectsLocationsServiceProjectAttachmentsListRequest(_messages.Message):
  r"""A ApphubProjectsLocationsServiceProjectAttachmentsListRequest object.

  Fields:
    filter: Optional. Filtering results.
    orderBy: Optional. Hint for how to order the results.
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Host project ID and location to list service project
      attachments. Only global location is supported. Expected format:
      `projects/{project}/locations/{location}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ApphubProjectsLocationsUpdateBoundaryRequest(_messages.Message):
  r"""A ApphubProjectsLocationsUpdateBoundaryRequest object.

  Fields:
    boundary: A Boundary resource to be passed as the request body.
    name: Identifier. The resource name of the boundary. Format:
      "projects/{project}/locations/{location}/boundary"
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the Boundary resource by the update. The fields specified
      in the update_mask are relative to the resource, not the full request. A
      field will be overwritten if it is in the mask. If the user does not
      provide a mask then all fields will be overwritten.
  """

  boundary = _messages.MessageField('Boundary', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class Application(_messages.Message):
  r"""Application defines the governance boundary for App Hub entities that
  perform a logical end-to-end business function. App Hub supports application
  level IAM permission to align with governance requirements.

  Enums:
    StateValueValuesEnum: Output only. Application state.

  Fields:
    attributes: Optional. Consumer provided attributes.
    createTime: Output only. Create time.
    description: Optional. User-defined description of an Application. Can
      have a maximum length of 2048 characters.
    displayName: Optional. User-defined name for the Application. Can have a
      maximum length of 63 characters.
    name: Identifier. The resource name of an Application. Format:
      `"projects/{host-project-
      id}/locations/{location}/applications/{application-id}"`
    scope: Required. Immutable. Defines what data can be included into this
      Application. Limits which Services and Workloads can be registered.
    state: Output only. Application state.
    uid: Output only. A universally unique identifier (in UUID4 format) for
      the `Application`.
    updateTime: Output only. Update time.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. Application state.

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      CREATING: The Application is being created.
      ACTIVE: The Application is ready to register Services and Workloads.
      DELETING: The Application is being deleted.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DELETING = 3

  attributes = _messages.MessageField('Attributes', 1)
  createTime = _messages.StringField(2)
  description = _messages.StringField(3)
  displayName = _messages.StringField(4)
  name = _messages.StringField(5)
  scope = _messages.MessageField('Scope', 6)
  state = _messages.EnumField('StateValueValuesEnum', 7)
  uid = _messages.StringField(8)
  updateTime = _messages.StringField(9)


class Attributes(_messages.Message):
  r"""Consumer provided attributes.

  Fields:
    businessOwners: Optional. Business team that ensures user needs are met
      and value is delivered
    criticality: Optional. User-defined criticality information.
    developerOwners: Optional. Developer team that owns development and
      coding.
    environment: Optional. User-defined environment information.
    operatorOwners: Optional. Operator team that ensures runtime and
      operations.
  """

  businessOwners = _messages.MessageField('ContactInfo', 1, repeated=True)
  criticality = _messages.MessageField('Criticality', 2)
  developerOwners = _messages.MessageField('ContactInfo', 3, repeated=True)
  environment = _messages.MessageField('Environment', 4)
  operatorOwners = _messages.MessageField('ContactInfo', 5, repeated=True)


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts `jose@example.com` from DATA_READ logging, and
  `aliya@example.com` from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class Binding(_messages.Message):
  r"""Associates `members`, or principals, with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the principals in this
      binding. To learn which resources support conditions in their IAM
      policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the principals requesting access for a Google Cloud
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. Does not include identities that come from
      external identity providers (IdPs) through identity federation. *
      `user:{emailid}`: An email address that represents a specific Google
      account. For example, `alice@example.com` . *
      `serviceAccount:{emailid}`: An email address that represents a Google
      service account. For example, `my-other-
      app@appspot.gserviceaccount.com`. *
      `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`:
      An identifier for a [Kubernetes service
      account](https://cloud.google.com/kubernetes-engine/docs/how-
      to/kubernetes-service-accounts). For example, `my-
      project.svc.id.goog[my-namespace/my-kubernetes-sa]`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. * `domain:{domain}`: The G Suite domain
      (primary) that represents all the users of that domain. For example,
      `google.com` or `example.com`. * `principal://iam.googleapis.com/locatio
      ns/global/workforcePools/{pool_id}/subject/{subject_attribute_value}`: A
      single identity in a workforce identity pool. * `principalSet://iam.goog
      leapis.com/locations/global/workforcePools/{pool_id}/group/{group_id}`:
      All workforce identities in a group. * `principalSet://iam.googleapis.co
      m/locations/global/workforcePools/{pool_id}/attribute.{attribute_name}/{
      attribute_value}`: All workforce identities with a specific attribute
      value. * `principalSet://iam.googleapis.com/locations/global/workforcePo
      ols/{pool_id}/*`: All identities in a workforce identity pool. * `princi
      pal://iam.googleapis.com/projects/{project_number}/locations/global/work
      loadIdentityPools/{pool_id}/subject/{subject_attribute_value}`: A single
      identity in a workload identity pool. * `principalSet://iam.googleapis.c
      om/projects/{project_number}/locations/global/workloadIdentityPools/{poo
      l_id}/group/{group_id}`: A workload identity pool group. * `principalSet
      ://iam.googleapis.com/projects/{project_number}/locations/global/workloa
      dIdentityPools/{pool_id}/attribute.{attribute_name}/{attribute_value}`:
      All identities in a workload identity pool with a certain attribute. * `
      principalSet://iam.googleapis.com/projects/{project_number}/locations/gl
      obal/workloadIdentityPools/{pool_id}/*`: All identities in a workload
      identity pool. * `deleted:user:{emailid}?uid={uniqueid}`: An email
      address (plus unique identifier) representing a user that has been
      recently deleted. For example,
      `alice@example.com?uid=123456789012345678901`. If the user is recovered,
      this value reverts to `user:{emailid}` and the recovered user retains
      the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `deleted:principal://iam.google
      apis.com/locations/global/workforcePools/{pool_id}/subject/{subject_attr
      ibute_value}`: Deleted single identity in a workforce identity pool. For
      example, `deleted:principal://iam.googleapis.com/locations/global/workfo
      rcePools/my-pool-id/subject/my-subject-attribute-value`.
    role: Role that is assigned to the list of `members`, or principals. For
      example, `roles/viewer`, `roles/editor`, or `roles/owner`. For an
      overview of the IAM roles and permissions, see the [IAM
      documentation](https://cloud.google.com/iam/docs/roles-overview). For a
      list of the available pre-defined roles, see
      [here](https://cloud.google.com/iam/docs/understanding-roles).
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class Boundary(_messages.Message):
  r"""Application management boundary.

  Enums:
    TypeValueValuesEnum: Output only. Boundary type.

  Fields:
    createTime: Output only. Create time.
    crmNode: Optional. The resource name of the CRM node being attached to the
      boundary. Format: `projects/{project-number}` or `projects/{project-id}`
    name: Identifier. The resource name of the boundary. Format:
      "projects/{project}/locations/{location}/boundary"
    type: Output only. Boundary type.
    updateTime: Output only. Update time.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Output only. Boundary type.

    Values:
      TYPE_UNSPECIFIED: Unspecified type.
      AUTOMATIC: The Boundary automatically includes all descendants of the
        CRM node.
      MANUAL: The list of projects within the Boundary is managed by the user.
      MANAGED_AUTOMATIC: The Boundary automatically includes all descendants
        of the CRM node, which is set via App Management folder capability.
    """
    TYPE_UNSPECIFIED = 0
    AUTOMATIC = 1
    MANUAL = 2
    MANAGED_AUTOMATIC = 3

  createTime = _messages.StringField(1)
  crmNode = _messages.StringField(2)
  name = _messages.StringField(3)
  type = _messages.EnumField('TypeValueValuesEnum', 4)
  updateTime = _messages.StringField(5)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class ContactInfo(_messages.Message):
  r"""Contact information of stakeholders.

  Fields:
    displayName: Optional. Contact's name. Can have a maximum length of 63
      characters.
    email: Required. Email address of the contacts.
  """

  displayName = _messages.StringField(1)
  email = _messages.StringField(2)


class Criticality(_messages.Message):
  r"""Criticality of the Application, Service, or Workload

  Enums:
    TypeValueValuesEnum: Required. Criticality Type.

  Fields:
    type: Required. Criticality Type.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Required. Criticality Type.

    Values:
      TYPE_UNSPECIFIED: Unspecified type.
      MISSION_CRITICAL: Mission critical service, application or workload.
      HIGH: High impact.
      MEDIUM: Medium impact.
      LOW: Low impact.
    """
    TYPE_UNSPECIFIED = 0
    MISSION_CRITICAL = 1
    HIGH = 2
    MEDIUM = 3
    LOW = 4

  type = _messages.EnumField('TypeValueValuesEnum', 1)


class DetachServiceProjectAttachmentRequest(_messages.Message):
  r"""Request for DetachServiceProjectAttachment."""


class DetachServiceProjectAttachmentResponse(_messages.Message):
  r"""Response for DetachServiceProjectAttachment."""


class DiscoveredService(_messages.Message):
  r"""DiscoveredService is a network or API interface that exposes some
  functionality to clients for consumption over the network. A discovered
  service can be registered to a App Hub service.

  Fields:
    name: Identifier. The resource name of the discovered service. Format:
      `"projects/{host-project-
      id}/locations/{location}/discoveredServices/{uuid}"`
    serviceProperties: Output only. Properties of an underlying compute
      resource that can comprise a Service. These are immutable.
    serviceReference: Output only. Reference to an underlying networking
      resource that can comprise a Service. These are immutable.
  """

  name = _messages.StringField(1)
  serviceProperties = _messages.MessageField('ServiceProperties', 2)
  serviceReference = _messages.MessageField('ServiceReference', 3)


class DiscoveredWorkload(_messages.Message):
  r"""DiscoveredWorkload is a binary deployment (such as managed instance
  groups (MIGs) and GKE deployments) that performs the smallest logical subset
  of business functionality. A discovered workload can be registered to an App
  Hub Workload.

  Fields:
    name: Identifier. The resource name of the discovered workload. Format:
      `"projects/{host-project-
      id}/locations/{location}/discoveredWorkloads/{uuid}"`
    workloadProperties: Output only. Properties of an underlying compute
      resource represented by the Workload. These are immutable.
    workloadReference: Output only. Reference of an underlying compute
      resource represented by the Workload. These are immutable.
  """

  name = _messages.StringField(1)
  workloadProperties = _messages.MessageField('WorkloadProperties', 2)
  workloadReference = _messages.MessageField('WorkloadReference', 3)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class Environment(_messages.Message):
  r"""Environment of the Application, Service, or Workload

  Enums:
    TypeValueValuesEnum: Required. Environment Type.

  Fields:
    type: Required. Environment Type.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Required. Environment Type.

    Values:
      TYPE_UNSPECIFIED: Unspecified type.
      PRODUCTION: Production environment.
      STAGING: Staging environment.
      TEST: Test environment.
      DEVELOPMENT: Development environment.
    """
    TYPE_UNSPECIFIED = 0
    PRODUCTION = 1
    STAGING = 2
    TEST = 3
    DEVELOPMENT = 4

  type = _messages.EnumField('TypeValueValuesEnum', 1)


class Expr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class ExtendedMetadata(_messages.Message):
  r"""Additional metadata for a Service or Workload.

  Messages:
    MetadataStructValue: Output only. The metadata contents.

  Fields:
    extendedMetadataSchema: Output only. The resource name for the Extended
      Metadata Schema that describes the contents of `metadata_struct`.
      Format: projects//locations//extendedMetadataSchemas/
    metadataStruct: Output only. The metadata contents.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataStructValue(_messages.Message):
    r"""Output only. The metadata contents.

    Messages:
      AdditionalProperty: An additional property for a MetadataStructValue
        object.

    Fields:
      additionalProperties: Properties of the object.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataStructValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  extendedMetadataSchema = _messages.StringField(1)
  metadataStruct = _messages.MessageField('MetadataStructValue', 2)


class ExtendedMetadataSchema(_messages.Message):
  r"""ExtendedMetadataSchema represents a schema for extended metadata of a
  service or workload.

  Fields:
    jsonSchema: Output only. The JSON schema as a string.
    name: Identifier. Resource name of the schema. Format:
      projects//locations//extendedMetadataSchemas/
    schemaVersion: Output only. The version of the schema. New versions are
      required to be backwards compatible.
  """

  jsonSchema = _messages.StringField(1)
  name = _messages.StringField(2)
  schemaVersion = _messages.IntegerField(3)


class FunctionalType(_messages.Message):
  r"""The functional type of a service or workload.

  Enums:
    TypeValueValuesEnum: Output only. The functional type of a service or
      workload.

  Fields:
    type: Output only. The functional type of a service or workload.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Output only. The functional type of a service or workload.

    Values:
      TYPE_UNSPECIFIED: Unspecified type.
      AGENT: Agent type.
      MCP_SERVER: MCP Server type.
      ENDPOINT: Endpoint type.
    """
    TYPE_UNSPECIFIED = 0
    AGENT = 1
    MCP_SERVER = 2
    ENDPOINT = 3

  type = _messages.EnumField('TypeValueValuesEnum', 1)


class Identity(_messages.Message):
  r"""The identity associated with a service or workload.

  Fields:
    principal: Output only. The principal of the identity. Supported formats:
      * `sa://my-sa@PROJECT_ID.iam.gserviceaccount.com` for GCP Service
      Account * `principal://POOL_ID.global.PROJECT_NUMBER.workload.id.goog/ns
      /NAMESPACE_ID/sa/MANAGED_IDENTITY_ID` for Managed Workload Identity
  """

  principal = _messages.StringField(1)


class ListApplicationsResponse(_messages.Message):
  r"""Response for ListApplications.

  Fields:
    applications: List of Applications.
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Locations that could not be reached.
  """

  applications = _messages.MessageField('Application', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListDiscoveredServicesResponse(_messages.Message):
  r"""Response for ListDiscoveredServices.

  Fields:
    discoveredServices: List of Discovered Services.
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Locations that could not be reached.
  """

  discoveredServices = _messages.MessageField('DiscoveredService', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListDiscoveredWorkloadsResponse(_messages.Message):
  r"""Response for ListDiscoveredWorkloads.

  Fields:
    discoveredWorkloads: List of Discovered Workloads.
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Locations that could not be reached.
  """

  discoveredWorkloads = _messages.MessageField('DiscoveredWorkload', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListExtendedMetadataSchemasResponse(_messages.Message):
  r"""Response for ListExtendedMetadataSchemas.

  Fields:
    extendedMetadataSchemas: List of Extended Metadata Schemas.
    nextPageToken: A token identifying a page of results the server should
      return.
  """

  extendedMetadataSchemas = _messages.MessageField('ExtendedMetadataSchema', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListServiceProjectAttachmentsResponse(_messages.Message):
  r"""Response for ListServiceProjectAttachments.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    serviceProjectAttachments: List of service project attachments.
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  serviceProjectAttachments = _messages.MessageField('ServiceProjectAttachment', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListServicesResponse(_messages.Message):
  r"""Response for ListServices.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    services: List of Services.
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  services = _messages.MessageField('Service', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListWorkloadsResponse(_messages.Message):
  r"""Response for ListWorkloads.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Locations that could not be reached.
    workloads: List of Workloads.
  """

  nextPageToken = _messages.StringField(1)
  unreachable = _messages.StringField(2, repeated=True)
  workloads = _messages.MessageField('Workload', 3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class LookupDiscoveredServiceResponse(_messages.Message):
  r"""Response for LookupDiscoveredService.

  Fields:
    discoveredService: Discovered Service if exists, empty otherwise.
  """

  discoveredService = _messages.MessageField('DiscoveredService', 1)


class LookupDiscoveredWorkloadResponse(_messages.Message):
  r"""Response for LookupDiscoveredWorkload.

  Fields:
    discoveredWorkload: Discovered Workload if exists, empty otherwise.
  """

  discoveredWorkload = _messages.MessageField('DiscoveredWorkload', 1)


class LookupServiceProjectAttachmentResponse(_messages.Message):
  r"""Response for LookupServiceProjectAttachment.

  Fields:
    serviceProjectAttachment: Service project attachment for a project if
      exists, empty otherwise.
  """

  serviceProjectAttachment = _messages.MessageField('ServiceProjectAttachment', 1)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have google.longrunning.Operation.error value
      with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members`, or principals, to a
  single `role`. Principals can be user accounts, service accounts, Google
  groups, and domains (such as G Suite). A `role` is a named list of
  permissions; each `role` can be an IAM predefined role or a user-created
  custom role. For some types of Google Cloud resources, a `binding` can also
  specify a `condition`, which is a logical expression that allows access to a
  resource only if the expression evaluates to `true`. A condition can add
  constraints based on attributes of the request, the resource, or both. To
  learn which resources support conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** ``` { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } ``` **YAML example:** ``` bindings: -
  members: - user:mike@example.com - group:admins@example.com -
  domain:google.com - serviceAccount:my-project-id@appspot.gserviceaccount.com
  role: roles/resourcemanager.organizationAdmin - members: -
  user:eve@example.com role: roles/resourcemanager.organizationViewer
  condition: title: expirable access description: Does not grant access after
  Sep 2020 expression: request.time < timestamp('2020-10-01T00:00:00.000Z')
  etag: BwWWja0YfJA= version: 3 ``` For a description of IAM and its features,
  see the [IAM documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members`, or principals, with a `role`.
      Optionally, may specify a `condition` that determines how and when the
      `bindings` are applied. Each of the `bindings` must contain at least one
      principal. The `bindings` in a `Policy` can refer to up to 1,500
      principals; up to 250 of these principals can be Google groups. Each
      occurrence of a principal counts towards these limits. For example, if
      the `bindings` grant 50 different roles to `user:alice@example.com`, and
      not to any other principal, then you can add another 1,450 principals to
      the `bindings` in the `Policy`.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class ReconciliationOperationMetadata(_messages.Message):
  r"""Operation metadata returned by the CLH during resource state
  reconciliation.

  Enums:
    ExclusiveActionValueValuesEnum: Excluisive action returned by the CLH.

  Fields:
    deleteResource: DEPRECATED. Use exclusive_action instead.
    exclusiveAction: Excluisive action returned by the CLH.
  """

  class ExclusiveActionValueValuesEnum(_messages.Enum):
    r"""Excluisive action returned by the CLH.

    Values:
      UNKNOWN_REPAIR_ACTION: Unknown repair action.
      DELETE: The resource has to be deleted. When using this bit, the CLH
        should fail the operation. DEPRECATED. Instead use DELETE_RESOURCE
        OperationSignal in SideChannel.
      RETRY: This resource could not be repaired but the repair should be
        tried again at a later time. This can happen if there is a dependency
        that needs to be resolved first- e.g. if a parent resource must be
        repaired before a child resource.
    """
    UNKNOWN_REPAIR_ACTION = 0
    DELETE = 1
    RETRY = 2

  deleteResource = _messages.BooleanField(1)
  exclusiveAction = _messages.EnumField('ExclusiveActionValueValuesEnum', 2)


class RegistrationType(_messages.Message):
  r"""The registration type of a service.

  Enums:
    TypeValueValuesEnum: Output only. The registration type of a service.

  Fields:
    type: Output only. The registration type of a service.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Output only. The registration type of a service.

    Values:
      TYPE_UNSPECIFIED: Unspecified registration type. Defaults to EXCLUSIVE.
      EXCLUSIVE: The service can only be registered to one application.
      SHARED: The service can be registered to multiple applications.
    """
    TYPE_UNSPECIFIED = 0
    EXCLUSIVE = 1
    SHARED = 2

  type = _messages.EnumField('TypeValueValuesEnum', 1)


class Scope(_messages.Message):
  r"""Scope of an application.

  Enums:
    TypeValueValuesEnum: Required. Scope Type.

  Fields:
    type: Required. Scope Type.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Required. Scope Type.

    Values:
      TYPE_UNSPECIFIED: Unspecified type.
      REGIONAL: Regional type.
      GLOBAL: Global type.
    """
    TYPE_UNSPECIFIED = 0
    REGIONAL = 1
    GLOBAL = 2

  type = _messages.EnumField('TypeValueValuesEnum', 1)


class Service(_messages.Message):
  r"""Service is an App Hub data model that contains a discovered service,
  which represents a network or API interface that exposes some functionality
  to clients for consumption over the network.

  Enums:
    StateValueValuesEnum: Output only. Service state.

  Fields:
    attributes: Optional. Consumer provided attributes.
    createTime: Output only. Create time.
    description: Optional. User-defined description of a Service. Can have a
      maximum length of 2048 characters.
    discoveredService: Required. Immutable. The resource name of the original
      discovered service.
    displayName: Optional. User-defined name for the Service. Can have a
      maximum length of 63 characters.
    name: Identifier. The resource name of a Service. Format:
      `"projects/{host-project-
      id}/locations/{location}/applications/{application-
      id}/services/{service-id}"`
    serviceProperties: Output only. Properties of an underlying compute
      resource that can comprise a Service. These are immutable.
    serviceReference: Output only. Reference to an underlying networking
      resource that can comprise a Service. These are immutable.
    state: Output only. Service state.
    uid: Output only. A universally unique identifier (UUID) for the `Service`
      in the UUID4 format.
    updateTime: Output only. Update time.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. Service state.

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      CREATING: The service is being created.
      ACTIVE: The service is ready.
      DELETING: The service is being deleted.
      DETACHED: The underlying networking resources have been deleted.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DELETING = 3
    DETACHED = 4

  attributes = _messages.MessageField('Attributes', 1)
  createTime = _messages.StringField(2)
  description = _messages.StringField(3)
  discoveredService = _messages.StringField(4)
  displayName = _messages.StringField(5)
  name = _messages.StringField(6)
  serviceProperties = _messages.MessageField('ServiceProperties', 7)
  serviceReference = _messages.MessageField('ServiceReference', 8)
  state = _messages.EnumField('StateValueValuesEnum', 9)
  uid = _messages.StringField(10)
  updateTime = _messages.StringField(11)


class ServiceProjectAttachment(_messages.Message):
  r"""ServiceProjectAttachment represents an attachment from a service project
  to a host project. Service projects contain the underlying cloud
  infrastructure resources, and expose these resources to the host project
  through a ServiceProjectAttachment. With the attachments, the host project
  can provide an aggregated view of resources across all service projects.

  Enums:
    StateValueValuesEnum: Output only. ServiceProjectAttachment state.

  Fields:
    createTime: Output only. Create time.
    name: Identifier. The resource name of a ServiceProjectAttachment. Format:
      `"projects/{host-project-
      id}/locations/global/serviceProjectAttachments/{service-project-id}."`
    serviceProject: Required. Immutable. Service project name in the format:
      `"projects/abc"` or `"projects/123"`. As input, project name with either
      project id or number are accepted. As output, this field will contain
      project number.
    state: Output only. ServiceProjectAttachment state.
    uid: Output only. A globally unique identifier (in UUID4 format) for the
      `ServiceProjectAttachment`.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. ServiceProjectAttachment state.

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      CREATING: The ServiceProjectAttachment is being created.
      ACTIVE: The ServiceProjectAttachment is ready. This means Services and
        Workloads under the corresponding ServiceProjectAttachment is ready
        for registration.
      DELETING: The ServiceProjectAttachment is being deleted.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DELETING = 3

  createTime = _messages.StringField(1)
  name = _messages.StringField(2)
  serviceProject = _messages.StringField(3)
  state = _messages.EnumField('StateValueValuesEnum', 4)
  uid = _messages.StringField(5)


class ServiceProperties(_messages.Message):
  r"""Properties of an underlying cloud resource that can comprise a Service.

  Messages:
    ExtendedMetadataValue: Output only. Additional metadata specific to the
      resource type. The key is a string that identifies the type of metadata
      and the value is the metadata contents specific to that type. Key
      format: `apphub.googleapis.com/{metadataType}`

  Fields:
    extendedMetadata: Output only. Additional metadata specific to the
      resource type. The key is a string that identifies the type of metadata
      and the value is the metadata contents specific to that type. Key
      format: `apphub.googleapis.com/{metadataType}`
    functionalType: Output only. The type of the service.
    gcpProject: Output only. The service project identifier that the
      underlying cloud resource resides in.
    identity: Output only. The identity associated with the service.
    location: Output only. The location that the underlying resource resides
      in, for example, us-west1.
    registrationType: Output only. The registration type of the service.
    zone: Output only. The location that the underlying resource resides in if
      it is zonal, for example, us-west1-a).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ExtendedMetadataValue(_messages.Message):
    r"""Output only. Additional metadata specific to the resource type. The
    key is a string that identifies the type of metadata and the value is the
    metadata contents specific to that type. Key format:
    `apphub.googleapis.com/{metadataType}`

    Messages:
      AdditionalProperty: An additional property for a ExtendedMetadataValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        ExtendedMetadataValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ExtendedMetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A ExtendedMetadata attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('ExtendedMetadata', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  extendedMetadata = _messages.MessageField('ExtendedMetadataValue', 1)
  functionalType = _messages.MessageField('FunctionalType', 2)
  gcpProject = _messages.StringField(3)
  identity = _messages.MessageField('Identity', 4)
  location = _messages.StringField(5)
  registrationType = _messages.MessageField('RegistrationType', 6)
  zone = _messages.StringField(7)


class ServiceReference(_messages.Message):
  r"""Reference to an underlying networking resource that can comprise a
  Service.

  Fields:
    uri: Output only. The underlying resource URI. For example, URI of
      Forwarding Rule, URL Map, and Backend Service.
  """

  uri = _messages.StringField(1)


class SetIamPolicyRequest(_messages.Message):
  r"""Request message for `SetIamPolicy` method.

  Fields:
    policy: REQUIRED: The complete policy to be applied to the `resource`. The
      size of the policy is limited to a few 10s of KB. An empty policy is a
      valid policy but certain Google Cloud services (such as Projects) might
      reject them.
    updateMask: OPTIONAL: A FieldMask specifying which fields of the policy to
      modify. Only the fields in the mask will be modified. If no mask is
      provided, the following default mask is used: `paths: "bindings, etag"`
  """

  policy = _messages.MessageField('Policy', 1)
  updateMask = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TestIamPermissionsRequest(_messages.Message):
  r"""Request message for `TestIamPermissions` method.

  Fields:
    permissions: The set of permissions to check for the `resource`.
      Permissions with wildcards (such as `*` or `storage.*`) are not allowed.
      For more information see [IAM
      Overview](https://cloud.google.com/iam/docs/overview#permissions).
  """

  permissions = _messages.StringField(1, repeated=True)


class TestIamPermissionsResponse(_messages.Message):
  r"""Response message for `TestIamPermissions` method.

  Fields:
    permissions: A subset of `TestPermissionsRequest.permissions` that the
      caller is allowed.
  """

  permissions = _messages.StringField(1, repeated=True)


class Workload(_messages.Message):
  r"""Workload is an App Hub data model that contains a discovered workload,
  which represents a binary deployment (such as managed instance groups (MIGs)
  and GKE deployments) that performs the smallest logical subset of business
  functionality.

  Enums:
    StateValueValuesEnum: Output only. Workload state.

  Fields:
    attributes: Optional. Consumer provided attributes.
    createTime: Output only. Create time.
    description: Optional. User-defined description of a Workload. Can have a
      maximum length of 2048 characters.
    discoveredWorkload: Required. Immutable. The resource name of the original
      discovered workload.
    displayName: Optional. User-defined name for the Workload. Can have a
      maximum length of 63 characters.
    name: Identifier. The resource name of the Workload. Format:
      `"projects/{host-project-
      id}/locations/{location}/applications/{application-
      id}/workloads/{workload-id}"`
    state: Output only. Workload state.
    uid: Output only. A universally unique identifier (UUID) for the
      `Workload` in the UUID4 format.
    updateTime: Output only. Update time.
    workloadProperties: Output only. Properties of an underlying compute
      resource represented by the Workload. These are immutable.
    workloadReference: Output only. Reference of an underlying compute
      resource represented by the Workload. These are immutable.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. Workload state.

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      CREATING: The Workload is being created.
      ACTIVE: The Workload is ready.
      DELETING: The Workload is being deleted.
      DETACHED: The underlying compute resources have been deleted.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DELETING = 3
    DETACHED = 4

  attributes = _messages.MessageField('Attributes', 1)
  createTime = _messages.StringField(2)
  description = _messages.StringField(3)
  discoveredWorkload = _messages.StringField(4)
  displayName = _messages.StringField(5)
  name = _messages.StringField(6)
  state = _messages.EnumField('StateValueValuesEnum', 7)
  uid = _messages.StringField(8)
  updateTime = _messages.StringField(9)
  workloadProperties = _messages.MessageField('WorkloadProperties', 10)
  workloadReference = _messages.MessageField('WorkloadReference', 11)


class WorkloadProperties(_messages.Message):
  r"""Properties of an underlying compute resource represented by the
  Workload.

  Messages:
    ExtendedMetadataValue: Output only. Additional metadata specific to the
      resource type. The key is a string that identifies the type of metadata
      and the value is the metadata contents specific to that type. Key
      format: `apphub.googleapis.com/{metadataType}`

  Fields:
    extendedMetadata: Output only. Additional metadata specific to the
      resource type. The key is a string that identifies the type of metadata
      and the value is the metadata contents specific to that type. Key
      format: `apphub.googleapis.com/{metadataType}`
    functionalType: Output only. The type of the workload.
    gcpProject: Output only. The service project identifier that the
      underlying cloud resource resides in. Empty for non-cloud resources.
    identity: Output only. The identity associated with the workload.
    location: Output only. The location that the underlying compute resource
      resides in (for example, us-west1).
    zone: Output only. The location that the underlying compute resource
      resides in if it is zonal (for example, us-west1-a).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ExtendedMetadataValue(_messages.Message):
    r"""Output only. Additional metadata specific to the resource type. The
    key is a string that identifies the type of metadata and the value is the
    metadata contents specific to that type. Key format:
    `apphub.googleapis.com/{metadataType}`

    Messages:
      AdditionalProperty: An additional property for a ExtendedMetadataValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        ExtendedMetadataValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ExtendedMetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A ExtendedMetadata attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('ExtendedMetadata', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  extendedMetadata = _messages.MessageField('ExtendedMetadataValue', 1)
  functionalType = _messages.MessageField('FunctionalType', 2)
  gcpProject = _messages.StringField(3)
  identity = _messages.MessageField('Identity', 4)
  location = _messages.StringField(5)
  zone = _messages.StringField(6)


class WorkloadReference(_messages.Message):
  r"""Reference of an underlying compute resource represented by the Workload.

  Fields:
    uri: Output only. The underlying compute resource uri.
  """

  uri = _messages.StringField(1)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
encoding.AddCustomJsonFieldMapping(
    ApphubProjectsLocationsApplicationsGetIamPolicyRequest, 'options_requestedPolicyVersion', 'options.requestedPolicyVersion')



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to restart migration jobs for a database migration."""


from googlecloudsdk.api_lib.database_migration import api_util
from googlecloudsdk.api_lib.database_migration import migration_jobs
from googlecloudsdk.api_lib.database_migration import resource_args
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log

DETAILED_HELP = {
    'DESCRIPTION': """
        Fetch objects for a Database Migration Service migration job by connection to the source.
        """,
    'EXAMPLES': """\
        To fetch source objects for a migration job:

          $ {command} MIGRATION_JOB --region=us-central1
        """,
}


class _MigrationJobObjectInfo:
  """Container for migration job object data using in list display."""

  def __init__(self, message):
    self.name = message.name
    self.source_object = message.sourceObject
    self.error = message.error if message.error is not None else None
    self.state = message.state
    self.phase = message.phase
    self.create_time = message.createTime
    self.update_time = message.updateTime


@base.RegionalEndpointsSupported
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class FetchSourceObjects(base.Command):
  """Fetch objects for a Database Migration Service migration job by connection to the source."""

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    resource_args.AddOnlyMigrationJobResourceArgs(parser, 'to restart')
    parser.display_info.AddFormat("""
            table(
              source_object,
              state:label=STATE,
              phase:label=PHASE,
              error:label=ERROR
            )
          """)

  def Run(self, args):
    """Fetch source objects for a Database Migration Service migration job.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A dict object representing the operations resource describing the restart
      operation if the restart was successful.
    """
    migration_job_ref = args.CONCEPTS.migration_job.Parse()
    location = migration_job_ref.locationsId

    mj_client = migration_jobs.MigrationJobsClient(
        self.ReleaseTrack(),
        location=location,
    )
    result_operation = mj_client.FetchSourceObjects(
        migration_job_ref.RelativeName(),
    )

    client = api_util.GetClientInstance(
        self.ReleaseTrack(),
        location=location,
    )

    log.status.Print(
        'Waiting for migration job [{}] to fetch source objects with [{}]'
        .format(migration_job_ref.migrationJobsId, result_operation.name)
    )

    api_util.HandleLRO(
        client,
        result_operation,
        client.projects_locations_migrationJobs,
        no_resource=True,
    )

    log.status.Print(
        'Fetched source objects for migration job {} [{}]'.format(
            migration_job_ref.migrationJobsId, result_operation.name
        )
    )

    obj = mj_client.ListObjects(migration_job_ref)

    return [_MigrationJobObjectInfo(o) for o in obj]



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Retry wrapper for resumable BiDi downloads."""

from __future__ import annotations

from typing import Any

from googlecloudsdk.api_lib.storage import errors as cloud_errors
from googlecloudsdk.api_lib.storage import retry_util as storage_retry_util
from googlecloudsdk.api_lib.storage.gcs_grpc import grpc_util
from googlecloudsdk.api_lib.storage.gcs_grpc import metadata_util
from googlecloudsdk.api_lib.storage.gcs_grpc import retry_util as grpc_retry_util
from googlecloudsdk.api_lib.storage.gcs_grpc_bidi_streaming import download
from googlecloudsdk.api_lib.storage.gcs_grpc_bidi_streaming import storage_bidi_rpc
from googlecloudsdk.command_lib.storage.resources import resource_reference
from googlecloudsdk.core import log
from googlecloudsdk.core.util import retry

_MAX_RETRIES_ON_REDIRECTED_TOKEN_ERROR = 4


class BidiUploadStreamClosedError(cloud_errors.RetryableApiError):
  """Exception raised when the BiDi upload stream is unexpectedly closed."""


def _should_retry_bidi(exc_type, exc_value, exc_traceback, state=None):
  """Returns True if the BiDi download error is retryable."""
  if isinstance(exc_value, BrokenPipeError):
    return False
  if isinstance(
      exc_value, download.BidiDownloadIncompleteError
  ) or grpc_retry_util.is_retriable(exc_type, exc_value, exc_traceback, state):
    log.debug('BiDi download interrupted by error, checking if retryable: %s',
              exc_value)
    return True
  return False


def run_with_retries(
    process_chunk_func,
    gapic_client,
    cloud_resource,
    download_stream,
    digesters,
    progress_callback,
    start_byte,
    end_byte,
    download_strategy,
    decryption_key,
    target_size,
    redirection_handler,
):
  """Executes download with retries, resuming from processed_bytes."""
  bidi_downloader = download.BidiDownloader(
      process_chunk_func,
      gapic_client,
      cloud_resource,
      download_stream,
      digesters,
      progress_callback,
      start_byte,
      end_byte,
      download_strategy,
      decryption_key,
      target_size,
      redirection_handler,
  )

  def _should_retry_bidi_with_reset(exc_type, exc_value, exc_traceback, state):
    """Wrapper for _should_retry_bidi that resets state if retry is needed."""
    should_retry = _should_retry_bidi(exc_type, exc_value, exc_traceback, state)
    if not should_retry:
      return False

    if isinstance(exc_value, download.BidiDownloadIncompleteError):
      # If download is incomplete, download_stream is consistent with
      # processed_bytes and digesters state because BidiDownloadIncompleteError
      # is raised after process_chunk_func returns. Thus, we can resume.
      log.debug('Resuming Bidi download from byte %s',
                bidi_downloader.processed_bytes)
    else:
      # If stream broke unexpectedly (e.g. gRPC error), reset stream state to
      # processed_bytes.
      if not bidi_downloader.download_stream.seekable():
        log.debug(
            'Cannot reset non-seekable stream for retry. Download will fail.'
        )
        return False
      log.debug(
          'Bidi stream failed unexpectedly. Resuming download for byte range'
          ' (%s, %s) from offset %s. Error: %s',
          bidi_downloader.start_byte,
          bidi_downloader.end_byte,
          bidi_downloader.processed_bytes,
          exc_value,
      )
      bidi_downloader.download_stream.seek(bidi_downloader.processed_bytes)
      bidi_downloader.read_handle = None
    return True

  try:
    storage_retry_util.retryer(
        target=bidi_downloader.download_chunk,
        should_retry_if=_should_retry_bidi_with_reset,
    )
  except (download.BidiDownloadIncompleteError, retry.MaxRetrialsException):
    # Retries exhausted.
    pass
  return (
      bidi_downloader.processed_bytes,
      bidi_downloader.destination_pipe_is_broken,
  )


class BidiRedirectedTokenErrorHandler:
  """Handles retries on redirected token errors for BiDi RPCs.

  This class is NOT thread-safe due to in-place modification of the
  `initial_request` object in `_get_bidi_rpc_with_routing_token`.

  A new instance of this handler should be used for independent data transfer
  operation (e.g., each new download or upload). However, the same
  class(routing_token) can be re-used for uploading-downloading the same object.
  The API discards stale tokens, ensuring that an incorrect token from a
  previous or different stream does not result in an incorrect behavior.

  Once created, the routing token remains valid for a few minutes only.

  Attributes:
    _client: The GAPIC client.
    _destination_resource: Metadata for the destination object.
    _routing_token: The routing token to be used for the BiDi RPC.
  """

  _UPLOADS_TYPE_URL = (
      # gcloud-disable-gdu-domain
      'type.googleapis.com/google.storage.v2.BidiWriteObjectRedirectedError'
  )
  _DOWNLOADS_TYPE_URL = (
      # gcloud-disable-gdu-domain
      'type.googleapis.com/google.storage.v2.BidiReadObjectRedirectedError'
  )

  def __init__(
      self,
      client,
      *,
      source_resource: Any | None = None,
      destination_resource: (
          resource_reference.ObjectResource | resource_reference.UnknownResource
      ) | None = None,
  ):
    """Initializes the BidiRedirectedTokenErrorHandler.

    Args:
      client (gapic_clients.storage_v2.services.storage.client.StorageClient):
        The GAPIC client.
      source_resource: The source resource of the data transfer.
      destination_resource: The destination resource of the data transfer.

    """
    self._client = client
    self._source_resource = source_resource
    self._destination_resource = destination_resource
    if (
        destination_resource is not None
        and isinstance(destination_resource, resource_reference.Resource)
        and hasattr(destination_resource, 'storage_url')
        and destination_resource.storage_url.bucket_name
    ):
      self._cloud_resource = destination_resource
    else:
      self._cloud_resource = source_resource
    self._routing_token = None

  def __repr__(self) -> str:
    return (
        f'{self.__class__.__name__}('
        f'client={self._client!r}, '
        f'source_resource={self._source_resource!r}, '
        f'destination_resource={self._destination_resource!r}, '
        f'cloud_resource={self._cloud_resource!r}, '
        f'routing_token={self._routing_token!r})'
    )

  def _get_bidi_rpc_with_routing_token(self, initial_request):
    """Gets the bidi rpc with routing token.

    Incase of redirected token error, the token received is to be provided in
    the header `x-goog-request-params`, with key `routing_token` and the token
    string verbatim as the value.

    We must also populate the initial request with the token, for uploads and
    downloads. Note that this method modifies `initial_request` in place.

    Args:
      initial_request: (gapic_clients.storage_v2.types.BidiWriteObjectRequest
        | gapic_clients.storage_v2.types.BidiReadObjectRequest) The initial
        request to be used for the bidi RPC.

    Returns:
      gapic_util.BidiRpc: The bidi RPC with metadata and initial request
        updated.
    """
    if isinstance(initial_request, self._client.types.BidiWriteObjectRequest):
      rpc_method = self._client.storage.bidi_write_object
      if initial_request.append_object_spec:
        initial_request.append_object_spec.routing_token = self._routing_token
    elif isinstance(initial_request, self._client.types.BidiReadObjectRequest):
      rpc_method = self._client.storage.bidi_read_object
      initial_request.read_object_spec.routing_token = self._routing_token
    else:
      raise ValueError(
          'Unsupported initial request type: %s' % type(initial_request)
      )

    return storage_bidi_rpc.StorageBidiRpc(
        self._client,
        rpc_method,
        initial_request=initial_request,
        metadata=metadata_util.get_bucket_name_routing_header(
            grpc_util.get_full_bucket_name(
                self._cloud_resource.storage_url.bucket_name
            ),
            routing_token=self._routing_token,
        ),
        source_resource=self._source_resource,
        destination_resource=self._destination_resource,
    )

  def _get_routing_token_from_redirected_error(
      self, error: grpc_retry_util.exceptions.Aborted
  ) -> str | None:
    """Fetches the routing token from a redirected error.

    Parses the details of a gRPC Aborted error to extract a routing token
    and updates `self._routing_token` with the new token.

    Args:
      error: The grpc.RpcError exception, expected to be an Aborted error
        containing details about a redirection.

    Raises:
      ValueError: If the error details do not contain a recognized redirected
        error type URL (e.g., BidiReadObjectRedirectedError or
        BidiWriteObjectRedirectedError).
      Exception: If there is an error during the deserialization of the
        redirected error.

    Returns:
      str | None: The routing token if found, otherwise None.
    """
    try:
      from google.api_core import protobuf_helpers  # pylint: disable=g-import-not-at-top
      for any_proto in error.details:
        # Retry only if the type_url matches the expected error type.
        if any_proto.type_url == self._DOWNLOADS_TYPE_URL:
          return protobuf_helpers.from_any_pb(
              self._client.types.BidiReadObjectRedirectedError, any_proto
          ).routing_token
        if any_proto.type_url == self._UPLOADS_TYPE_URL:
          return protobuf_helpers.from_any_pb(
              self._client.types.BidiWriteObjectRedirectedError, any_proto
          ).routing_token
      return None
    except (TypeError, ImportError) as e:
      log.debug('Failed to process the any proto from Aborted error: %s', e)
      raise

  def start_bidi_rpc_with_retry_on_redirected_token_error(
      self, initial_request
  ):
    """Starts the bidi rpc with retry on redirected token error."""
    for retry_count in range(_MAX_RETRIES_ON_REDIRECTED_TOKEN_ERROR):
      bidi_rpc = self._get_bidi_rpc_with_routing_token(initial_request)
      try:
        bidi_rpc.open()
        return bidi_rpc
      except grpc_retry_util.exceptions.Aborted as e:
        log.debug(
            'Aborted error during BiDi RPC for data transfer from %s to %s: %s',
            self._source_resource,
            self._destination_resource,
            e,
        )
        bidi_rpc.close()
        if retry_count == _MAX_RETRIES_ON_REDIRECTED_TOKEN_ERROR - 1:
          # If we have exhausted all retries, re-raise the error.
          raise
        self._routing_token = self._get_routing_token_from_redirected_error(e)
        if self._routing_token is None:
          # If the error is not a redirected error / routing token was not found
          # in the error(unexpected), re-raise the original error.
          raise
